#!/bin/sh
# the next line restarts this script using wish found in the path\
exec wish "$0" "$@"
# If this does not work, change the #!/usr/bin/wish line below
# to reflect the actual wish location and delete all preceeding lines
#
# (delete here and above)
#!/usr/bin/wish
#
# $Id: indexCIFdict.tcl,v 1.5 2003/07/25 00:33:47 toby Exp toby $

# where is this file running from?
set script [info script]
# translate links -- go six levels deep
foreach i {1 2 3 4 5 6} {
    if {[file type $script] == "link"} {
	set link [file readlink $script]
	if { [file  pathtype  $link] == "absolute" } {
	    set script $link
	} {
	    set script [file dirname $script]/$link
	}
    } else {
	break
    }
}
# fixup relative paths
if {[file pathtype $script] == "relative"} {
    set script [file join [pwd] $script]
}
set scriptdir [file dirname $script]

# where to find the BWidget program
lappend auto_path $scriptdir

# load the support library
source [file join $scriptdir browsecif.tcl]

if {[string first $argv -h] != -1} {
    set msg "Use [file tail $argv0] to create an index to one or more CIF dictionaries. "
    append msg "This routine writes a file named CIF_index in directory $scriptdir"
    append msg "\nusage:\n\t$argv0 CIFdic1.cif \[CIFdic2.cif\] \[...\]"
    MyMessageBox -parent . -title "Program info" \
		-message $msg -icon info -type OK -default OK
    exit
}

set filelist [split $argv " "]

if [catch {
    set fp [open [file join $scriptdir CIF_index] w]
}] {
    MyMessageBox -parent . -title "Write Error" \
		-message "Unable to write file [file join $scriptdir CIF_index], sorry." \
		-icon error -type "{C'est la vie}"
    exit
}
wm withdraw .
catch {destroy .file}
toplevel .file
wm title .file "CIF dictionary files"

proc AddDictFile {} {
    global filelist
    set file [tk_getOpenFile -title "Select CIF dictionary file"\
	    -parent .file \
	    -defaultextension dic -filetypes \
	    {{"CIF dictionary" .dic} {"All files" *.*}}]
    if {$file == ""} return
    if {![file exists $file]} return
    lappend filelist $file
    eval destroy [winfo children .file.f]
    set row 0
    grid [label .file.f.0 -text "Dictionary file list:" -anchor w] \
	    -row 0 -sticky w
    foreach file $filelist {
	incr row
	grid [label .file.f.$row -text $file -anchor e] -row $row -sticky e
    }
    .file.b.c config -state normal
}
grid [frame .file.f -bd 4 -relief groove] -row 1 -column 1 -sticky ew
grid [frame .file.b] -row 2 -column 1
grid [button .file.b.a -text "Add file\nto list" \
	-command "AddDictFile"] -row 2 -column 0 -sticky ns
grid [button .file.b.c -text Continue -state disabled \
	-command "destroy .file"] -row 2 -column 1 -sticky ns
grid [button .file.b.q -text Quit \
	-command "exit"] -row 2 -column 2 -sticky ns

set newfilelist {}
set row 0
grid [label .file.f.0 -text "Dictionary file list:" -anchor w] -row 0 -sticky w
foreach file $filelist {
    incr row
    set file [file join [pwd] $file]
    # remove ..\ from path
    while {[set i [lsearch [set ep [file split $file]] ..]] != -1} {
	set file [eval file join [lreplace $ep [expr $i-1] $i]]
    }
    if {[file exists $file]} {
	grid [label .file.f.$row -text $file -anchor e] -row $row -sticky e
	lappend newfilelist $file
    }
}
set filelist $newfilelist
if {[llength $filelist] == 0} {
    grid [label .file.f.n -text "(no files)" -anchor center] -row $row -sticky ew
} else {
    .file.b.c config -state normal
}

tkwait window .file

catch {destroy .file}
toplevel .file
wm title .file "CIF file contents"
set txt .file.t
grid [text $txt -width 80 -yscrollcommand ".file.s set"] -column 0 -row 0
grid [scrollbar .file.s -command "$txt yview"] -column 1 -row 0 -sticky ns
wm withdraw .file

wm deiconify .
pack [label .l1 -text "Definitions Processed"] -side left
pack [label .l2 -textvariable parsestatus] -side left
update

set pathlist {}
puts $fp "array set CIF_dataname_index {"
set counter 0
foreach f $filelist {
    set dir [file dirname $f]
    if {[file pathtype $dir] == "relative"} {
	set dir [file join [pwd] $dir]
    }
    if {[lsearch $pathlist $dir] == -1} {
	lappend pathlist $dir
    }
    set parsestatus "$counter (clearing up)" 
    update
    # this is too slow:
    #$txt configure -state normal
    #$txt delete 1.0 end
    # this is faster(!):
    destroy $txt
    grid [text $txt -width 80 -yscrollcommand ".file.s set"] -column 0 -row 0

    set file [file tail $f]
    set parsestatus "$counter (reading $file)" 
    update
    set inp [open $f r]
    fconfigure $inp -translation binary
    set text [read $inp]
    # is this a DDL2 dictionary (with save frames)
    if {[string match -nocase "*save_*" $text]} {
	set DDL2 1
	regsub -all "save__" $text "data__" text
	regsub -all "save_" $text "####_" text
    } else {
	set DDL2 0
    }
    $txt insert end $text
    close $inp
    set parsestatus "$counter (starting parse)" 
    update
    set blocks [ParseCIF $txt]
    set allblocks {}
    set prevpos 1.0
    set prevbytes 0
    set parsestatus "$counter (parse complete)" 
    update
    for {set i 1} {$i <= $blocks} {incr i} {
	incr counter
	if {$counter % 10 == 0} {
	    set parsestatus $counter
	    update
	}
	lappend allblocks $i
	if {![catch {set block${i}(errors)} errmsg]} {
	    puts stderr "Block $i ([set block${i}(data_)]) errors:"
	    puts stderr "[set block${i}(errors)]"
	}
	# list of positions for dataname
	set list {}
	catch {set list [set block${i}(_name)]}
	if {$list == "" && $DDL2} { 
	    catch {set list [set block${i}(_item.name)]}
	}
	# definition entry
	set def {}
	catch {set def  [set block${i}(_definition)]}
	if {$def == "" && $DDL2} { 
	    catch {set def  [set block${i}(_item_description.description)]}
	} 
	if {$def == ""} continue
	if {[llength $def] != 1} {puts stderr "problem with [set block${i}(data_)]"}
	# count the number of bytes from the previous position
	# (much faster than counting from the beginning each time)
	#set defpos [string length [$txt get 1.0 $def.l]]
	set defpos [string length [$txt get $prevpos $def.l]]
	incr defpos $prevbytes
	set prevpos $def.l
	set prevbytes $defpos
	set deflen [string length [$txt get $def.l $def.r]]
	# item type (numb/char/null)
	set type {}
	catch {set type [set block${i}(_type)]}
	if {$type == "" && $DDL2} {
	    catch {set type [set block${i}(_item_type.code)]}
	    if {[llength $type] != 1} {
		set typeval "?"
	    } else {
		set typeval [StripQuotes [$txt get $type.l $type.r]]
	    }
	    # mmCIF uses: atcode, code, float, int, line, symop, text
	    #             uchar1, uchar3, ucode, uline, yyyy-mm-dd
	    # treat everything but float & int as character
	    if {$typeval == "float" || $typeval == "int"} {
		set typeval "n"
	    } else {
		set typeval "c"
	    }
	} elseif {[llength $type] != 1} {
	    puts stderr "type problem for [set block${i}(data_)]"
	    set typeval "?"
	} else {
	    set typeval [StripQuotes [$txt get $type.l $type.r]]
	    if {$typeval == "numb"} {
		set typeval "n"
	    } elseif {$typeval == "char"} {
		set typeval "c"
	    } elseif {$typeval == "null"} {
		set typeval ""
	    } else {
		puts stderr "Block [set block${i}(data_)] has invalid _type ($typeval)"
		set typeval "?"
	    }
	}
	# flag if esd's are allowed
	set pos {}
	catch {set pos [set block${i}(_type_conditions)]}
	if {$pos == "" && $DDL2} { 
	    catch {set pos [set block${i}(_item_type_conditions.code)]}
	}
	if {[llength $pos] != 1} {
	    set esd 0
	} else {
	    if {"esd" == [string tolower \
		    [StripQuotes [$txt get $pos.l $pos.r]]]} {set esd 1}
	}
	# units (_units_details overrides _units)
	set pos {}
	catch {set pos [set block${i}(_units)]}
	if {$pos == "" && $DDL2} {
	    catch {set pos [set block${i}(_item_units.code)]}
	} else {
	    catch {set pos [set block${i}(_units_details)]}
	}
	if {[llength $pos] != 1} {
	    set units {}
	} else {
	    set units [StripQuotes [$txt get $pos.l $pos.r]]
	}
	# parse out _enumeration _enumeration_detail & _enumeration_range
	set elist ""
	set enumlist {}
	set enumdetaillist {}
	if {$DDL2} {
	    catch {
		set enumlist [set block${i}(_item_enumeration.value)]
		set enumdetaillist [set block${i}(_item_enumeration.detail)]
	    }
	} else {
	    catch {
		set enumlist [set block${i}(_enumeration)]
		set enumdetaillist [set block${i}(_enumeration_detail)]
	    }
	}
	catch {
	    foreach m1 $enumlist \
		    m2 $enumdetaillist {
		if {$m2 != ""} {
		    set detail [StripQuotes [$txt get $m2.l $m2.r]]]
		    # condense multiple spaces out
		    regsub -all {  +} $detail { } detail
		} else {
		    set detail {}
		}
		lappend elist [list [StripQuotes [$txt get $m1.l $m1.r]] $detail]
	    }
	}
	# mmCIF ranges are too complex to do here
	set range ""
	catch {
	    set mark [set block${i}(_enumeration_range)] 
	    lappend range [StripQuotes [$txt get $mark.l $mark.r]]
	}

	set pos ""
	catch {set pos [set block${i}(_category)]}
	if {$pos == "" && $DDL2} {
	    catch {set pos [set block${i}(_item.category_id)]}
	}
	if {[llength $pos] != 1} {
	    set category {}
	} else {
	    set category [StripQuotes [$txt get $pos.l $pos.r]]
	}

	foreach mark $list {
	    set dataname [string tolower [StripQuotes [$txt get $mark.l $mark.r]]]
	    puts $fp "\t$dataname \t[list \
		    [list [list $file $defpos $deflen] \
		    [list $typeval $range $elist $esd $units $category]]]"
	}
    }
    set parsestatus "$counter (flush buffer)"
    update
    flush $fp
    update idletasks
}
puts $fp "}"
puts $fp "\nlappend CIF(cif_path) $pathlist"
close $fp
exit
